/*
 * Copyright (c) 2010 - 2011, The MiCode Open Source Community (www.micode.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *        http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

// 导入包声明，指定该类所在的包为 net.micode.notes.data
package net.micode.notes.data;

// 导入 Android 系统中用于存储键值对的 ContentValues 类
import android.content.ContentValues;
// 导入 Android 系统中代表应用程序上下文的 Context 类
import android.content.Context;
// 导入 Android 系统中用于操作 SQLite 数据库的 SQLiteDatabase 类
import android.database.sqlite.SQLiteDatabase;
// 导入 Android 系统中用于创建和管理 SQLite 数据库的辅助类 SQLiteOpenHelper
import android.database.sqlite.SQLiteOpenHelper;
// 导入 Android 系统中用于日志记录的 Log 类
import android.util.Log;

// 导入 Notes 类中的 DataColumns 内部类，用于引用数据列的常量
import net.micode.notes.data.Notes.DataColumns;
// 导入 Notes 类中的 DataConstants 内部类，用于引用数据相关的常量
import net.micode.notes.data.Notes.DataConstants;
// 导入 Notes 类中的 NoteColumns 内部类，用于引用笔记列的常量
import net.micode.notes.data.Notes.NoteColumns;

/**
 * NotesDatabaseHelper 类继承自 SQLiteOpenHelper，用于管理笔记应用的 SQLite 数据库。
 * 它负责创建、升级数据库，定义表结构和触发器，并提供对数据库操作的辅助方法。
 */
public class NotesDatabaseHelper extends SQLiteOpenHelper {
    // 定义数据库的名称为 note.db
    private static final String DB_NAME = "note.db";
    // 定义数据库的版本号为 4
    private static final int DB_VERSION = 4;

    /**
     * TABLE 接口定义了数据库中表的名称常量。
     */
    public interface TABLE {
        // 定义笔记表的名称为 note
        public static final String NOTE = "note";
        // 定义数据表的名称为 data
        public static final String DATA = "data";
    }

    // 定义日志标签，用于在 Log 中标识该类的日志信息
    private static final String TAG = "NotesDatabaseHelper";
    // 定义一个静态的 NotesDatabaseHelper 实例，用于实现单例模式
    private static NotesDatabaseHelper mInstance;

    // 定义创建笔记表的 SQL 语句
    private static final String CREATE_NOTE_TABLE_SQL =
            "CREATE TABLE " + TABLE.NOTE + "(" +
                    // 定义笔记的 ID 列，作为主键，类型为整数
                    NoteColumns.ID + " INTEGER PRIMARY KEY," +
                    // 定义笔记的父 ID 列，类型为整数，不可为空，默认值为 0
                    NoteColumns.PARENT_ID + " INTEGER NOT NULL DEFAULT 0," +
                    // 定义笔记的提醒日期列，类型为整数，不可为空，默认值为 0
                    NoteColumns.ALERTED_DATE + " INTEGER NOT NULL DEFAULT 0," +
                    // 定义笔记的背景颜色 ID 列，类型为整数，不可为空，默认值为 0
                    NoteColumns.BG_COLOR_ID + " INTEGER NOT NULL DEFAULT 0," +
                    // 定义笔记的创建日期列，类型为整数，不可为空，默认值为当前时间戳
                    NoteColumns.CREATED_DATE + " INTEGER NOT NULL DEFAULT (strftime('%s','now') * 1000)," +
                    // 定义笔记是否有附件列，类型为整数，不可为空，默认值为 0
                    NoteColumns.HAS_ATTACHMENT + " INTEGER NOT NULL DEFAULT 0," +
                    // 定义笔记的修改日期列，类型为整数，不可为空，默认值为当前时间戳
                    NoteColumns.MODIFIED_DATE + " INTEGER NOT NULL DEFAULT (strftime('%s','now') * 1000)," +
                    // 定义笔记的数量列，类型为整数，不可为空，默认值为 0
                    NoteColumns.NOTES_COUNT + " INTEGER NOT NULL DEFAULT 0," +
                    // 定义笔记的摘要列，类型为文本，不可为空，默认值为空字符串
                    NoteColumns.SNIPPET + " TEXT NOT NULL DEFAULT ''," +
                    // 定义笔记的类型列，类型为整数，不可为空，默认值为 0
                    NoteColumns.TYPE + " INTEGER NOT NULL DEFAULT 0," +
                    // 定义笔记的小部件 ID 列，类型为整数，不可为空，默认值为 0
                    NoteColumns.WIDGET_ID + " INTEGER NOT NULL DEFAULT 0," +
                    // 定义笔记的小部件类型列，类型为整数，不可为空，默认值为 -1
                    NoteColumns.WIDGET_TYPE + " INTEGER NOT NULL DEFAULT -1," +
                    // 定义笔记的同步 ID 列，类型为整数，不可为空，默认值为 0
                    NoteColumns.SYNC_ID + " INTEGER NOT NULL DEFAULT 0," +
                    // 定义笔记的本地修改标记列，类型为整数，不可为空，默认值为 0
                    NoteColumns.LOCAL_MODIFIED + " INTEGER NOT NULL DEFAULT 0," +
                    // 定义笔记的原始父 ID 列，类型为整数，不可为空，默认值为 0
                    NoteColumns.ORIGIN_PARENT_ID + " INTEGER NOT NULL DEFAULT 0," +
                    // 定义笔记的 Google 任务 ID 列，类型为文本，不可为空，默认值为空字符串
                    NoteColumns.GTASK_ID + " TEXT NOT NULL DEFAULT ''," +
                    // 定义笔记的版本列，类型为整数，不可为空，默认值为 0
                    NoteColumns.VERSION + " INTEGER NOT NULL DEFAULT 0" +
                    ")";

    // 定义创建数据表的 SQL 语句
    private static final String CREATE_DATA_TABLE_SQL =
            "CREATE TABLE " + TABLE.DATA + "(" +
                    // 定义数据的 ID 列，作为主键，类型为整数
                    DataColumns.ID + " INTEGER PRIMARY KEY," +
                    // 定义数据的 MIME 类型列，类型为文本，不可为空
                    DataColumns.MIME_TYPE + " TEXT NOT NULL," +
                    // 定义数据所属笔记的 ID 列，类型为整数，不可为空，默认值为 0
                    DataColumns.NOTE_ID + " INTEGER NOT NULL DEFAULT 0," +
                    // 定义数据的创建日期列，类型为整数，不可为空，默认值为当前时间戳
                    NoteColumns.CREATED_DATE + " INTEGER NOT NULL DEFAULT (strftime('%s','now') * 1000)," +
                    // 定义数据的修改日期列，类型为整数，不可为空，默认值为当前时间戳
                    NoteColumns.MODIFIED_DATE + " INTEGER NOT NULL DEFAULT (strftime('%s','now') * 1000)," +
                    // 定义数据的内容列，类型为文本，不可为空，默认值为空字符串
                    DataColumns.CONTENT + " TEXT NOT NULL DEFAULT ''," +
                    // 定义数据的第一个数据列，类型为整数
                    DataColumns.DATA1 + " INTEGER," +
                    // 定义数据的第二个数据列，类型为整数
                    DataColumns.DATA2 + " INTEGER," +
                    // 定义数据的第三个数据列，类型为文本，不可为空，默认值为空字符串
                    DataColumns.DATA3 + " TEXT NOT NULL DEFAULT ''," +
                    // 定义数据的第四个数据列，类型为文本，不可为空，默认值为空字符串
                    DataColumns.DATA4 + " TEXT NOT NULL DEFAULT ''," +
                    // 定义数据的第五个数据列，类型为文本，不可为空，默认值为空字符串
                    DataColumns.DATA5 + " TEXT NOT NULL DEFAULT ''" +
                    ")";

    // 定义在数据表上创建笔记 ID 索引的 SQL 语句
    private static final String CREATE_DATA_NOTE_ID_INDEX_SQL =
            "CREATE INDEX IF NOT EXISTS note_id_index ON " +
                    TABLE.DATA + "(" + DataColumns.NOTE_ID + ");";

    /**
     * 定义一个触发器，当更新笔记的父 ID 时，增加目标文件夹的笔记数量。
     */
    private static final String NOTE_INCREASE_FOLDER_COUNT_ON_UPDATE_TRIGGER =
            "CREATE TRIGGER increase_folder_count_on_update " +
                    " AFTER UPDATE OF " + NoteColumns.PARENT_ID + " ON " + TABLE.NOTE +
                    " BEGIN " +
                    // 更新目标文件夹的笔记数量，使其加 1
                    "  UPDATE " + TABLE.NOTE +
                    "   SET " + NoteColumns.NOTES_COUNT + "=" + NoteColumns.NOTES_COUNT + " + 1" +
                    "  WHERE " + NoteColumns.ID + "=new." + NoteColumns.PARENT_ID + ";" +
                    " END";

    /**
     * 定义一个触发器，当更新笔记的父 ID 时，减少原文件夹的笔记数量。
     */
    private static final String NOTE_DECREASE_FOLDER_COUNT_ON_UPDATE_TRIGGER =
            "CREATE TRIGGER decrease_folder_count_on_update " +
                    " AFTER UPDATE OF " + NoteColumns.PARENT_ID + " ON " + TABLE.NOTE +
                    " BEGIN " +
                    // 更新原文件夹的笔记数量，使其减 1，但前提是数量大于 0
                    "  UPDATE " + TABLE.NOTE +
                    "   SET " + NoteColumns.NOTES_COUNT + "=" + NoteColumns.NOTES_COUNT + "-1" +
                    "  WHERE " + NoteColumns.ID + "=old." + NoteColumns.PARENT_ID +
                    "  AND " + NoteColumns.NOTES_COUNT + ">0" + ";" +
                    " END";

    /**
     * 定义一个触发器，当插入新笔记时，增加目标文件夹的笔记数量。
     */
    private static final String NOTE_INCREASE_FOLDER_COUNT_ON_INSERT_TRIGGER =
            "CREATE TRIGGER increase_folder_count_on_insert " +
                    " AFTER INSERT ON " + TABLE.NOTE +
                    " BEGIN " +
                    // 更新目标文件夹的笔记数量，使其加 1
                    "  UPDATE " + TABLE.NOTE +
                    "   SET " + NoteColumns.NOTES_COUNT + "=" + NoteColumns.NOTES_COUNT + " + 1" +
                    "  WHERE " + NoteColumns.ID + "=new." + NoteColumns.PARENT_ID + ";" +
                    " END";

    /**
     * 定义一个触发器，当删除笔记时，减少原文件夹的笔记数量。
     */
    private static final String NOTE_DECREASE_FOLDER_COUNT_ON_DELETE_TRIGGER =
            "CREATE TRIGGER decrease_folder_count_on_delete " +
                    " AFTER DELETE ON " + TABLE.NOTE +
                    " BEGIN " +
                    // 更新原文件夹的笔记数量，使其减 1，但前提是数量大于 0
                    "  UPDATE " + TABLE.NOTE +
                    "   SET " + NoteColumns.NOTES_COUNT + "=" + NoteColumns.NOTES_COUNT + "-1" +
                    "  WHERE " + NoteColumns.ID + "=old." + NoteColumns.PARENT_ID +
                    "  AND " + NoteColumns.NOTES_COUNT + ">0;" +
                    " END";

    /**
     * 定义一个触发器，当插入类型为 {@link DataConstants#NOTE} 的数据时，更新笔记的摘要。
     */
    private static final String DATA_UPDATE_NOTE_CONTENT_ON_INSERT_TRIGGER =
            "CREATE TRIGGER update_note_content_on_insert " +
                    " AFTER INSERT ON " + TABLE.DATA +
                    " WHEN new." + DataColumns.MIME_TYPE + "='" + DataConstants.NOTE + "'" +
                    " BEGIN" +
                    // 更新笔记的摘要为新插入数据的内容
                    "  UPDATE " + TABLE.NOTE +
                    "   SET " + NoteColumns.SNIPPET + "=new." + DataColumns.CONTENT +
                    "  WHERE " + NoteColumns.ID + "=new." + DataColumns.NOTE_ID + ";" +
                    " END";

    /**
     * 定义一个触发器，当更新类型为 {@link DataConstants#NOTE} 的数据时，更新笔记的摘要。
     */
    private static final String DATA_UPDATE_NOTE_CONTENT_ON_UPDATE_TRIGGER =
            "CREATE TRIGGER update_note_content_on_update " +
                    " AFTER UPDATE ON " + TABLE.DATA +
                    " WHEN old." + DataColumns.MIME_TYPE + "='" + DataConstants.NOTE + "'" +
                    " BEGIN" +
                    // 更新笔记的摘要为更新后数据的内容
                    "  UPDATE " + TABLE.NOTE +
                    "   SET " + NoteColumns.SNIPPET + "=new." + DataColumns.CONTENT +
                    "  WHERE " + NoteColumns.ID + "=new." + DataColumns.NOTE_ID + ";" +
                    " END";

    /**
     * 定义一个触发器，当删除类型为 {@link DataConstants#NOTE} 的数据时，清空笔记的摘要。
     */
    private static final String DATA_UPDATE_NOTE_CONTENT_ON_DELETE_TRIGGER =
            "CREATE TRIGGER update_note_content_on_delete " +
                    " AFTER delete ON " + TABLE.DATA +
                    " WHEN old." + DataColumns.MIME_TYPE + "='" + DataConstants.NOTE + "'" +
                    " BEGIN" +
                    // 清空笔记的摘要
                    "  UPDATE " + TABLE.NOTE +
                    "   SET " + NoteColumns.SNIPPET + "=''" +
                    "  WHERE " + NoteColumns.ID + "=old." + DataColumns.NOTE_ID + ";" +
                    " END";

    /**
     * 定义一个触发器，当删除笔记时，删除该笔记相关的数据。
     */
    private static final String NOTE_DELETE_DATA_ON_DELETE_TRIGGER =
            "CREATE TRIGGER delete_data_on_delete " +
                    " AFTER DELETE ON " + TABLE.NOTE +
                    " BEGIN" +
                    // 删除与被删除笔记 ID 相同的数据
                    "  DELETE FROM " + TABLE.DATA +
                    "   WHERE " + DataColumns.NOTE_ID + "=old." + NoteColumns.ID + ";" +
                    " END";

    /**
     * 定义一个触发器，当删除文件夹时，删除该文件夹下的所有笔记。
     */
    private static final String FOLDER_DELETE_NOTES_ON_DELETE_TRIGGER =
            "CREATE TRIGGER folder_delete_notes_on_delete " +
                    " AFTER DELETE ON " + TABLE.NOTE +
                    " BEGIN" +
                    // 删除父 ID 等于被删除文件夹 ID 的所有笔记
                    "  DELETE FROM " + TABLE.NOTE +
                    "   WHERE " + NoteColumns.PARENT_ID + "=old." + NoteColumns.ID + ";" +
                    " END";

    /**
     * 定义一个触发器，当文件夹被移动到回收站时，将该文件夹下的所有笔记也移动到回收站。
     */
    private static final String FOLDER_MOVE_NOTES_ON_TRASH_TRIGGER =
            "CREATE TRIGGER folder_move_notes_on_trash " +
                    " AFTER UPDATE ON " + TABLE.NOTE +
                    " WHEN new." + NoteColumns.PARENT_ID + "=" + Notes.ID_TRASH_FOLER +
                    " BEGIN" +
                    // 将父 ID 等于被移动文件夹 ID 的所有笔记的父 ID 也设置为回收站 ID
                    "  UPDATE " + TABLE.NOTE +
                    "   SET " + NoteColumns.PARENT_ID + "=" + Notes.ID_TRASH_FOLER +
                    "  WHERE " + NoteColumns.PARENT_ID + "=old." + NoteColumns.ID + ";" +
                    " END";

    /**
     * 构造函数，用于创建 NotesDatabaseHelper 实例。
     *
     * @param context 应用程序上下文
     */
    public NotesDatabaseHelper(Context context) {
        // 调用父类 SQLiteOpenHelper 的构造函数，传入上下文、数据库名称、游标工厂和数据库版本
        super(context, DB_NAME, null, DB_VERSION);
    }

    /**
     * 创建笔记表，并重新创建相关触发器，同时创建系统文件夹。
     *
     * @param db SQLite 数据库实例
     */
    public void createNoteTable(SQLiteDatabase db) {
        // 执行创建笔记表的 SQL 语句
        db.execSQL(CREATE_NOTE_TABLE_SQL);
        // 重新创建笔记表的触发器
        reCreateNoteTableTriggers(db);
        // 创建系统文件夹
        createSystemFolder(db);
        // 记录日志，表示笔记表已创建
        Log.d(TAG, "note table has been created");
    }

    /**
     * 重新创建笔记表的触发器。
     *
     * @param db SQLite 数据库实例
     */
    private void reCreateNoteTableTriggers(SQLiteDatabase db) {
        // 删除所有相关的旧触发器
        db.execSQL("DROP TRIGGER IF EXISTS increase_folder_count_on_update");
        db.execSQL("DROP TRIGGER IF EXISTS decrease_folder_count_on_update");
        db.execSQL("DROP TRIGGER IF EXISTS decrease_folder_count_on_delete");
        db.execSQL("DROP TRIGGER IF EXISTS delete_data_on_delete");
        db.execSQL("DROP TRIGGER IF EXISTS increase_folder_count_on_insert");
        db.execSQL("DROP TRIGGER IF EXISTS folder_delete_notes_on_delete");
        db.execSQL("DROP TRIGGER IF EXISTS folder_move_notes_on_trash");

        // 创建新的触发器
        db.execSQL(NOTE_INCREASE_FOLDER_COUNT_ON_UPDATE_TRIGGER);
        db.execSQL(NOTE_DECREASE_FOLDER_COUNT_ON_UPDATE_TRIGGER);
        db.execSQL(NOTE_DECREASE_FOLDER_COUNT_ON_DELETE_TRIGGER);
        db.execSQL(NOTE_DELETE_DATA_ON_DELETE_TRIGGER);
        db.execSQL(NOTE_INCREASE_FOLDER_COUNT_ON_INSERT_TRIGGER);
        db.execSQL(FOLDER_DELETE_NOTES_ON_DELETE_TRIGGER);
        db.execSQL(FOLDER_MOVE_NOTES_ON_TRASH_TRIGGER);
    }

    /**
     * 创建系统文件夹，包括通话记录文件夹、根文件夹、临时文件夹和回收站文件夹。
     *
     * @param db SQLite 数据库实例
     */
    private void createSystemFolder(SQLiteDatabase db) {
        // 创建 ContentValues 对象，用于存储要插入的数据
        ContentValues values = new ContentValues();

        /**
         * 创建通话记录文件夹，用于存储通话笔记
         */
        // 设置通话记录文件夹的 ID
        values.put(NoteColumns.ID, Notes.ID_CALL_RECORD_FOLDER);
        // 设置通话记录文件夹的类型为系统文件夹
        values.put(NoteColumns.TYPE, Notes.TYPE_SYSTEM);
        // 插入通话记录文件夹到笔记表
        db.insert(TABLE.NOTE, null, values);

        /**
         * 创建根文件夹，作为默认文件夹
         */
        // 清空 ContentValues 对象
        values.clear();
        // 设置根文件夹的 ID
        values.put(NoteColumns.ID, Notes.ID_ROOT_FOLDER);
        // 设置根文件夹的类型为系统文件夹
        values.put(NoteColumns.TYPE, Notes.TYPE_SYSTEM);
        // 插入根文件夹到笔记表
        db.insert(TABLE.NOTE, null, values);

        /**
         * 创建临时文件夹，用于移动笔记
         */
        // 清空 ContentValues 对象
        values.clear();
        // 设置临时文件夹的 ID
        values.put(NoteColumns.ID, Notes.ID_TEMPARAY_FOLDER);
        // 设置临时文件夹的类型为系统文件夹
        values.put(NoteColumns.TYPE, Notes.TYPE_SYSTEM);
        // 插入临时文件夹到笔记表
        db.insert(TABLE.NOTE, null, values);

        /**
         * 创建回收站文件夹
         */
        // 清空 ContentValues 对象
        values.clear();
        // 设置回收站文件夹的 ID
        values.put(NoteColumns.ID, Notes.ID_TRASH_FOLER);
        // 设置回收站文件夹的类型为系统文件夹
        values.put(NoteColumns.TYPE, Notes.TYPE_SYSTEM);
        // 插入回收站文件夹到笔记表
        db.insert(TABLE.NOTE, null, values);
    }

    /**
     * 创建数据表，并重新创建相关触发器，同时创建笔记 ID 索引。
     *
     * @param db SQLite 数据库实例
     */
    public void createDataTable(SQLiteDatabase db) {
        // 执行创建数据表的 SQL 语句
        db.execSQL(CREATE_DATA_TABLE_SQL);
        // 重新创建数据表的触发器
        reCreateDataTableTriggers(db);
        // 执行创建笔记 ID 索引的 SQL 语句
        db.execSQL(CREATE_DATA_NOTE_ID_INDEX_SQL);
        // 记录日志，表示数据表已创建
        Log.d(TAG, "data table has been created");
    }

    /**
     * 重新创建数据表的触发器。
     *
     * @param db SQLite 数据库实例
     */
    private void reCreateDataTableTriggers(SQLiteDatabase db) {
        // 删除所有相关的旧触发器
        db.execSQL("DROP TRIGGER IF EXISTS update_note_content_on_insert");
        db.execSQL("DROP TRIGGER IF EXISTS update_note_content_on_update");
        db.execSQL("DROP TRIGGER IF EXISTS update_note_content_on_delete");

        // 创建新的触发器
        db.execSQL(DATA_UPDATE_NOTE_CONTENT_ON_INSERT_TRIGGER);
        db.execSQL(DATA_UPDATE_NOTE_CONTENT_ON_UPDATE_TRIGGER);
        db.execSQL(DATA_UPDATE_NOTE_CONTENT_ON_DELETE_TRIGGER);
    }

    /**
     * 获取 NotesDatabaseHelper 的单例实例。
     *
     * @param context 应用程序上下文
     * @return NotesDatabaseHelper 实例
     */
    static synchronized NotesDatabaseHelper getInstance(Context context) {
        // 如果实例为空，则创建一个新的实例
        if (mInstance == null) {
            mInstance = new NotesDatabaseHelper(context);
        }
        // 返回实例
        return mInstance;
    }

    /**
     * 当数据库首次创建时调用，用于创建笔记表和数据表。
     *
     * @param db SQLite 数据库实例
     */
    @Override
    public void onCreate(SQLiteDatabase db) {
        // 创建笔记表
        createNoteTable(db);
        // 创建数据表
        createDataTable(db);
    }

    /**
     * 当数据库版本更新时调用，用于升级数据库。
     *
     * @param db         SQLite 数据库实例
     * @param oldVersion 旧的数据库版本号
     * @param newVersion 新的数据库版本号
     */
    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        // 标记是否需要重新创建触发器
        boolean reCreateTriggers = false;
        // 标记是否跳过从版本 2 到版本 3 的升级
        boolean skipV2 = false;

        // 如果旧版本为 1，则升级到版本 2
        if (oldVersion == 1) {
            upgradeToV2(db);
            // 标记跳过从版本 2 到版本 3 的升级
            skipV2 = true;
            // 旧版本号加 1
            oldVersion++;
        }

        // 如果旧版本为 2 且未跳过升级，则升级到版本 3
        if (oldVersion == 2 && !skipV2) {
            upgradeToV3(db);
            // 标记需要重新创建触发器
            reCreateTriggers = true;
            // 旧版本号加 1
            oldVersion++;
        }

        // 如果旧版本为 3，则升级到版本 4
        if (oldVersion == 3) {
            upgradeToV4(db);
            // 旧版本号加 1
            oldVersion++;
        }

        // 如果需要重新创建触发器，则重新创建笔记表和数据表的触发器
        if (reCreateTriggers) {
            reCreateNoteTableTriggers(db);
            reCreateDataTableTriggers(db);
        }

        // 如果升级后版本号不等于新版本号，则抛出异常
        if (oldVersion != newVersion) {
            throw new IllegalStateException("Upgrade notes database to version " + newVersion
                    + " fails");
        }
    }

    /**
     * 将数据库从版本 1 升级到版本 2。
     *
     * @param db SQLite 数据库实例
     */
    private void upgradeToV2(SQLiteDatabase db) {
        // 删除笔记表
        db.execSQL("DROP TABLE IF EXISTS " + TABLE.NOTE);
        // 删除数据表
        db.execSQL("DROP TABLE IF EXISTS " + TABLE.DATA);
        // 重新创建笔记表
        createNoteTable(db);
        // 重新创建数据表
        createDataTable(db);
    }

    /**
     * 将数据库从版本 2 升级到版本 3。
     *
     * @param db SQLite 数据库实例
     */
    private void upgradeToV3(SQLiteDatabase db) {
        // 删除无用的触发器
        db.execSQL("DROP TRIGGER IF EXISTS update_note_modified_date_on_insert");
        db.execSQL("DROP TRIGGER IF EXISTS update_note_modified_date_on_delete");
        db.execSQL("DROP TRIGGER IF EXISTS update_note_modified_date_on_update");
        // 在笔记表中添加 Google 任务 ID 列
        db.execSQL("ALTER TABLE " + TABLE.NOTE + " ADD COLUMN " + NoteColumns.GTASK_ID
                + " TEXT NOT NULL DEFAULT ''");
        // 创建 ContentValues 对象，用于存储要插入的数据
        ContentValues values = new ContentValues();
        // 设置回收站文件夹的 ID
        values.put(NoteColumns.ID, Notes.ID_TRASH_FOLER);
        // 设置回收站文件夹的类型为系统文件夹
        values.put(NoteColumns.TYPE, Notes.TYPE_SYSTEM);
        // 插入回收站文件夹到笔记表
        db.insert(TABLE.NOTE, null, values);
    }

    /**
     * 将数据库从版本 3 升级到版本 4。
     *
     * @param db SQLite 数据库实例
     */
    private void upgradeToV4(SQLiteDatabase db) {
        // 在笔记表中添加版本列
        db.execSQL("ALTER TABLE " + TABLE.NOTE + " ADD COLUMN " + NoteColumns.VERSION
                + " INTEGER NOT NULL DEFAULT 0");
    }
}